import java.io.IOException;
import java.util.Vector;
import javax.bluetooth.BluetoothStateException;
import javax.bluetooth.DeviceClass;
import javax.bluetooth.DiscoveryAgent;
import javax.bluetooth.DiscoveryListener;
import javax.bluetooth.LocalDevice;
import javax.bluetooth.RemoteDevice;
import javax.bluetooth.ServiceRecord;
import javax.bluetooth.UUID;

public class BTUtils implements DiscoveryListener {

//---- Bluetooth attributes ----
    protected UUID uuid = new UUID(0x1101); // serial port profile
    protected int inquiryMode = DiscoveryAgent.GIAC; // no pairing is needed
    protected int connectionOptions = ServiceRecord.NOAUTHENTICATE_NOENCRYPT;

//---- Echo loop attributes ----
    protected int stopToken = 255;

//---- GUI attributes ----
    protected Vector deviceList = new Vector();
    protected ServiceRecord[] sr = new ServiceRecord[16];
    int sc = 0;

    public void BTUtils() {};

    public void startDeviceInquiry()
    {
        try
        {
            log("Start inquiry method - this will take few seconds...");
            DiscoveryAgent agent = getAgent();
            agent.startInquiry(inquiryMode, this);
	} catch (Exception e) {
            log(e);
	}
    }

    public void deviceDiscovered(RemoteDevice btDevice, DeviceClass cod)
    {
        log("A device discovered (" + getDeviceStr(btDevice) + ")");
	deviceList.addElement(btDevice);
    }

    public void inquiryCompleted(int discType)
    {
        log("Inquiry compeleted. Please select device from combo box.");
        M.onenqcmp();
    }

    //-------  Service search section -------

    public void startServiceSearch(int i) {
        try {
                RemoteDevice device = (RemoteDevice)deviceList.elementAt(i);
                log("Start search for Serial Port Profile service from "
		+ getDeviceStr(device));
                UUID uuids[] = new UUID[] { uuid };
		getAgent().searchServices(null, uuids, device, this);
            } catch (Exception e) {
                log(e);
            }
    }

	/**
	 * This method is called when a service(s) are discovered.This method starts
	 * a thread that handles the data exchange with the server.
	 */
    public void servicesDiscovered(int transId, ServiceRecord[] records)
    {
        log("Service discovered.");
        sc = records.length;
        sr = records;
	/*for (int i = 0; i < records.length; i++)
        {
            ServiceRecord rec = records[i];
            String url = rec.getConnectionURL(connectionOptions, false);
            //handleConnection(url);
	}*/
    }

    public void serviceSearchCompleted(int transID, int respCode)
    {
        String msg = null;
	switch (respCode)
        {
            case SERVICE_SEARCH_COMPLETED:
                msg = "the service search completed normally";
		break;
            case SERVICE_SEARCH_TERMINATED:
		msg = "the service search request was cancelled by a call to DiscoveryAgent.cancelServiceSearch()";
		break;
            case SERVICE_SEARCH_ERROR:
		msg = "an error occurred while processing the request";
		break;
            case SERVICE_SEARCH_NO_RECORDS:
		msg = "no records were found during the service search";
		break;
            case SERVICE_SEARCH_DEVICE_NOT_REACHABLE:
		msg = "the device specified in the search request could not be reached or the local device could not establish a connection to the remote device";
		break;
	}
            log("Service search completed - " + msg);
            M.onsvcsc();
	}

        public int devcnt()
        {
            return deviceList.size();
        }

        public int svccnt()
        {
            return sc;
        }

        public String devstr(int i)
        {
         return getDeviceStr((RemoteDevice) deviceList.elementAt(i));
        }

        public String svcurl(int i)
        {
            return sr[i].getConnectionURL(connectionOptions, false);
        }

    synchronized private void log(String msg)
    {
        M.onlog(msg);
    }

    private void log(Throwable e)
    {
        log(e.getMessage());
    }

	//-------  Utils section - contains utility functions -------

	private DiscoveryAgent getAgent() {
		try {
			return LocalDevice.getLocalDevice().getDiscoveryAgent();
		} catch (BluetoothStateException e) {
			throw new Error(e.getMessage());
		}
	}

	private String getDeviceStr(RemoteDevice btDevice) {
		return getFriendlyName(btDevice) + " - 0x"
				+ btDevice.getBluetoothAddress();
	}

	private String getFriendlyName(RemoteDevice btDevice) {
		try {
			return btDevice.getFriendlyName(false);
		} catch (IOException e) {
			return "no name available";
		}
	}


}